unit DemoRemoteControl;
//
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, REMOTECONTROL, AdvAppStyler, ActnList, Menus, StdCtrls, ExtCtrls,
  ComCtrls, Buttons, TeEngine, Series, TeeProcs, Chart;

type
  TfrmMain = class(TForm)
    ActionList: TActionList;
    MainMenu: TMainMenu;
    aConnect: TAction;
    aDisconnect: TAction;
    Connect1: TMenuItem;
    aDisconnect1: TMenuItem;
    Timer1sec: TTimer;
    StatusBar: TStatusBar;
    lbStatus: TListBox;
    lbModules: TListBox;
    edTRIG1delay: TEdit;
    btStart: TBitBtn;
    btStop: TBitBtn;
    aStart: TAction;
    aStop: TAction;
    btProgramTRIG1: TButton;
    aProgramTRIG1: TAction;
    Label1: TLabel;
    Chart2: TChart;
    Series1: TFastLineSeries;
    procedure aConnectExecute(Sender: TObject);
    procedure aDisconnectExecute(Sender: TObject);
    procedure Timer1secTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure aStartExecute(Sender: TObject);
    procedure aStopExecute(Sender: TObject);
    procedure aProgramTRIG1Execute(Sender: TObject);
    procedure edTRIG1delayKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
  const
  MAX_string = 128;
  type
  NameType = array [0..MAX_string-1] of AnsiChar ;
  public
    { Public declarations }
  const
  DefaultTimeout =20;
  SyncModuleName: NameType = 'SY3PL50M:32';    // Replace with actual name !!!
  PHD2ModuleName: NameType = 'PHD1K000:49';    // Replace with actual name !!!
  var
  Connected: boolean;
  function ErrorCodeToString(Code:Integer; PopUpMessage:boolean):String;
  procedure GetModulesList;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.aConnectExecute(Sender: TObject);
var
Code:integer;
Timestamp: Integer;
ParameterValue:Double;
begin
Code:= rcConnect(0,0);
ErrorCodeToString(Code, True);
if Code = 0 then
  begin
  Connected:=true;
  StatusBar.Panels[0].Text:='Connected';
  MainMenu.Items[0].Enabled:=false;
  MainMenu.Items[1].Enabled:=true;
  GetModulesList;
  rcGetRegAsDouble (@SyncModuleName,'OUT3 delay', ParameterValue , DefaultTimeout, timestamp);
  edTRIG1delay.Text:= FloatToStr(ParameterValue);
  end;
end;

procedure TfrmMain.aDisconnectExecute(Sender: TObject);
begin
rcDisconnect;
Connected:=false;
StatusBar.Panels[0].Text:='Disconnected';
StatusBar.Panels[1].Text:='';
MainMenu.Items[0].Enabled:=true;
MainMenu.Items[1].Enabled:=false;
end;

procedure TfrmMain.aProgramTRIG1Execute(Sender: TObject);
begin
rcSetRegNVFromDouble(@SyncModuleName, 'OUT3 delay', StrToFloat(edTRIG1delay.Text));
end;

procedure TfrmMain.aStartExecute(Sender: TObject);
begin
rcSetRegFromString (@SyncModuleName,'State','ON');
end;

procedure TfrmMain.aStopExecute(Sender: TObject);
begin
rcSetRegFromString (@SyncModuleName,'State','OFF');
end;

procedure TfrmMain.edTRIG1delayKeyPress(Sender: TObject; var Key: Char);
begin
if Key = #13 then
  rcSetRegFromDouble(@SyncModuleName, 'OUT3 delay', StrToFloat(edTRIG1delay.Text));
end;

function TfrmMain.ErrorCodeToString(Code:Integer; PopUpMessage:boolean):String;
const
  ErrorList: Array[0..18] of string = ('Success, no error',
'End of registers list','No config file found' ,'Wrong CSV file'
,'Application provided return buffer is too short','No such device name',
'No such register name','Failed to connect','Timeout waiting for device answer','Register is read only'
,'Register is not NV','Attempt to set value above upper limit'
,'Attempt to set value below bottom limit','Attempt to set not allowed value','Register is not being logged'
,'Not enough memory','No data in the queue yet','Already connected, please disconnect first'
,'Not connected, please connect first');
begin
if PopUpMessage and (Code > 0) then
      ShowMessage(ErrorList[Code])
else ErrorCodeToString:= ErrorList[Code];
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
Connected:=false;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
if Connected then  rcDisconnect;
end;

procedure TfrmMain.Timer1secTimer(Sender: TObject);      // Parameter polling
var
Value: array [0..MAX_string-1] of AnsiChar;
Timestamp: Integer;
Code:integer;
I:integer;
ParameterValue:Double;
const
ListLength = 6;
ModuleList: Array [0..ListLength-1] of NameType = ('SY3PL50M:32','SY3PL50M:32','SY3PL50M:32','SY3PL50M:32','PHD1K000:48','PHD1K000:49');
RegisterList: Array [0..ListLength-1] of PAnsiChar =
  ('State','OUT3 delay','PRE-T delay','Continuous / Burst mode / Trigger burst','Mean level','Mean level');
RegisterDescriptionList: Array [0..ListLength-1] of PAnsiChar =
  ('Laser status: ','TRIG1 delay: ','PRET delay (11.11ns units): ','Burst mode: ','Regen output: ','Amplifier output: ');
begin
if Connected then
  begin
  for I := 0 to ListLength - 1 do
    begin
    Code:=rcGetRegAsString(@ModuleList[I],RegisterList[I], @Value, MAX_string ,DefaultTimeout, Timestamp);
    if Code=0 then lbStatus.Items[I] := RegisterDescriptionList[I] + String(Value);
    end;
  StatusBar.Panels[1].Text:= ErrorCodeToString(Code, False);
  Code:=rcGetRegAsDouble (@PHD2ModuleName,'Mean level', ParameterValue , DefaultTimeout, timestamp);   //getting value far a chart
  Series1.AddY(ParameterValue);
  end;
end;

procedure TfrmMain.GetModulesList;
var
Value: array [0..MAX_string-1] of AnsiChar;
Code:integer;
begin
Code:=rcGetFirstDeviceName(@value,MAX_string);
if Code =0 then
begin
  lbModules.Items.Clear;
  lbModules.Items.Add(String(Value));
  while rcGetNextDeviceName(@value,MAX_string)=0 do lbModules.Items.Add(String(Value));
end;


end;

end.
